# Generated by Django 4.2.18 on 2025-03-13 14:03

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("slack_messages", "0005_alter_slackmessagingpolicy_template_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="slackmessage",
            name="id",
            field=models.CharField(
                default=uuid.uuid4,
                editable=False,
                max_length=255,
                primary_key=True,
                serialize=False,
                unique=True,
            ),
        ),
    ]
